#include <stdio.h>

#define src argv[1]
#define dst argv[2]
#define name argv[3]
#define varHeaderSize 19		// 17 bytes for the variable header + 2 bytes for the size prefix on appvars

#define error(x)	{				\
						printf(x);	\
						return(0);	\
					}


int main(int argc, char *argv[])
{
  FILE *fp;
  unsigned int fileSize,sum = 0,i;
  unsigned char buff[0xFFFF], checksum[2];
  unsigned char header[55] = {
		0x2A, 0x2A, 0x54, 0x49, 0x38, 0x33, 0x46, 0x2A,		// **TI83F*
		0x1A, 0x0A,	0x00,									// signature

		0x41, 0x70, 0x70, 0x56, 0x61, 0x72, 0x69, 0x61,		// comment area
		0x62, 0x6c, 0x65, 0x20, 0x66, 0x69, 0x6c, 0x65,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
		0x00, 0x00, 

		0x00, 0x00											// data size
  };
  unsigned char varheader[varHeaderSize] = {
		0x0D,0x00,											// start of variable header
		0x00,0x00,											// length of variable in bytes
		0x15,												// variable type ID. 0x15 is for appvar
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,			// variable name (max 8 characters)
		0x00,												// version
		0x00,												// flag. 80h for archived variable. 00h otherwise
		0x00,0x00,											// length of variable in bytes (copy)
		0x00,0x00											// length of variable in bytes (another copy...)
  };

  /*-----------------------------------------*/
  // Too little parameters?
  if (argc<4)
	  error("Error: Not enough parameters. Usage: [source file] [destination file] [name(max 8 chars)]\n Max fileSize: 65516 bytes");
  // Name too long?
  if (strlen(name)>8)
	  error("_Name_Too_Long_Error");

  // Check if file can be opened (exists/in use)
  if ((fp = fopen(src, "rb"))==EOF)
	  error("_SourceFile_Open_Error");
  // Calculate source fileSize.
  fseek(fp, 0, SEEK_END);
  fileSize = ftell(fp);
  printf("File Size = %d\n",fileSize);
  fseek(fp, 0, SEEK_SET); 
  // File too big?
  if (fileSize > 0xFFEC)
	  error("_File_Size_Error(MAX64k)");
  fread(buff,1,fileSize,fp);
  // Check if file was closed correctly.
  if (fclose(fp)==EOF)
	  error("_SourceFile_Close_Error");
  /*__________________________________________*/

  // Remove destination file if it already exists.
  remove(dst);
  // Create destination file and open for appending data.
  if ((fp = fopen(dst, "ab"))==EOF)
	  error("_Target_Open_Error");
   /*__________________________________________*/
  // Write header to target file.
  header[53] = ((fileSize + varHeaderSize)) & 0xFF;		// Data Size + Var header size (L)
  header[54] = ((fileSize + varHeaderSize)>>8) & 0xFF;	// Data Size + Var header size (H)
  fwrite(header,1,55,fp);
  // Write variable header to target file.
  varheader[2] =  (fileSize + 2) & 0xFF;				// Data Size (L)
  varheader[3] =  ((fileSize + 2)>>8) & 0xFF;			// Data Size (H)

  // Copy name of variable.
  for(i=0; i<8; i++)
	varheader[5+i] = name[i];
  varheader[15] = (fileSize + 2) & 0xFF;				// Data Size (L)
  varheader[16] = ((fileSize + 2)>>8) & 0xFF;			// Data Size (H)
  varheader[17] = fileSize & 0xFF;						// Data Size (L)
  varheader[18] = (fileSize>>8) & 0xFF;					// Data Size (H)
  fwrite(varheader,1,varHeaderSize,fp);

  // Write variable data to target file.
  fwrite(buff,1,fileSize,fp);

  // Calculate and Write checksum to target file.
  for(i = 0; i<fileSize; i++)
    sum += buff[i];
  for(i = 0; i<varHeaderSize; i++)
    sum += varheader[i];
  checksum[0] = (sum) & 0xFF;		// Checksum (L)
  checksum[1] = (sum>>8) & 0xFF;	// Checksum (H)
  fwrite(checksum,1,2,fp);
   /*__________________________________________*/
  if (fclose(fp)==EOF)
	  error("_Target_Close_Error");

  printf("File converted succesfully!\n");
  
  return 0;
}